#!/bin/bash
#
# Script di creazione di pacchetti RPM per Gambas2.
#
# Author: Luigi Carlotto <luigi.carlotto@mclink.it>
#

PrjDir=$(pwd)
Macro="$HOME/.rpmmacros"
GambasVersion=`cat $PrjDir/GAMBAS.VERSION`

save_macro() {
  if [ -f $Macro ]; then
    mv $Macro $Macro.save
  fi
}

kill_macro() {
  if [ -f $Macro ]; then
    rm $Macro
  fi
}

restore_macro() {
  kill_macro
  if [ -f $Macro.save ]; then
    mv $Macro.save $Macro
  fi
}

cd ..

if [ ! -f $PrjDir/.project ]; then
  echo ".project not found."
  exit 1
fi

PrjTitle=`cat $PrjDir/.project | grep "Title=" | awk -F"=" '{printf $2}'`
PrjStartup=`cat $PrjDir/.project | grep "Startup=" | awk -F"=" '{printf $2}'`
PrjIcon=`cat $PrjDir/.project | grep "Icon=" | awk -F"=" '{printf $2}'`
PrjMajorVers=`cat $PrjDir/.project | grep "Version=" | awk -F"=" '{printf $2}' | awk -F"." '{printf $1}'`
PrjMinorVers=`cat $PrjDir/.project | grep "Version=" | awk -F"=" '{printf $2}' | awk -F"." '{printf $2}'`
PrjRelVers=`cat $PrjDir/.project | grep "Version=" | awk -F"=" '{printf $2}' | awk -F"." '{printf $3}'`
PrjDescription=`cat $PrjDir/.project | grep "Description=" | awk -F"=" '{printf $2}'`
PrjAuthor=`cat $PrjDir/.project | grep "Author=" | awk -F"=" '{printf $2}'`
PrjTabSize=`cat $PrjDir/.project | grep "TabSize=" | awk -F"=" '{printf $2}'`
PrjTranslate=`cat $PrjDir/.project | grep "Translate=" | awk -F"=" '{printf $2}'`
PrjKeepDebugInfo=`cat $PrjDir/.project | grep "KeepDebugInfo=" | awk -F"=" '{printf $2}'`
PrjControlPublic=`cat $PrjDir/.project | grep "ControlPublic=" | awk -F"=" '{printf $2}'`
PrjMaintainer=`cat $PrjDir/.project | grep "Maintainer=" | awk -F"=" '{printf $2}'`
PrjAddress="md9327@mclink.it" #`cat $PrjDir/.project | grep "Address=" | awk -F"=" '{printf $2}'`
PrjLicense=`cat $PrjDir/.project | grep "License=" | awk -F"=" '{printf $2}'`
PrjSystems=`cat $PrjDir/.project | grep "Systems=" | awk -F"=" '{printf $2}'`
PrjMenus=`cat $PrjDir/.project | grep "Menus=" | awk -F"=" '{printf $2}'`
PrjCategories=`cat $PrjDir/.project | grep "Categories=" | awk -F"=" '{printf $2}'`
PrjGroups=`cat $PrjDir/.project | grep "Groups=" | awk -F"=" '{printf $2}'`
if [ "`cat $PrjDir/.project | grep "Prefix=" | awk -F"=" '{printf $2}'`" = "1" ]; then
  PrjPrefix="gambas-"
else
  PrjPrefix=""
fi
PrjFilename="$PrjPrefix$PrjStartup.gambas"

which rpmbuild > /dev/null
if [ $? -eq 0 ]; then
  RPMBUILD_PROG="rpmbuild"
else
  which rpm > /dev/null
  if [ $? -eq 0 ]; then
    RPMBUILD_PROG="rpm"
  else
    echo "rpmbuild is not installed on your system."
    exit 1
  fi
fi

case "$PrjSystems" in
  mandrake)
    Distribution="MandrakeLinux"
    Vendor="MandrakeSoft"
    VersionSuffix="mdk"
    ;;
  debian)
    Distribution="Debian"
    Vendor="Debian"
    VersionSuffix=""
    ;;
  redhat)
    Distribution="RedHat"
    Vendor="RedHat"
    VersionSuffix=""
    ;;
  suse)
    Distribution="SuSE Linux"
    Vendor="SuSE Linux AG"
    VersionSuffix="suse"
    ;;
  fedora)
    Distribution="Fedora"
    Vendor="Fedora"
    VersionSuffix=""
    ;;
  *)
    echo "Distribution not defined."
    exit 1
    ;;
esac

echo "Creating package for $PrjStartup"

echo "Initializing ~/RPM directory."

for Path in RPM/ RPM/BUILD/ RPM/RPMS/ RPM/SOURCES/ RPM/SPECS/ RPM/SRPMS/ RPM/tmp/; do
  if [ ! -d $HOME/$Path ]; then
    mkdir $HOME/$Path > /dev/null
  fi
done

save_macro

(
echo "%_topdir $HOME/RPM"
echo "%_tmppath $HOME/RPM/tmp"
echo "%distribution $Distribution"
echo "%vendor $Vendor"
echo "%packager $PrjAuthor < $PrjAddress >"
echo "%buildhost http://pgdesigner.sourceforge.net"
) > $HOME/.rpmmacros

echo "Creating source package."

if [ ! -d $PrjDir/.icon ]; then
  mkdir $PrjDir/.icon > /dev/null
else
  rm -f $PrjDir/.icon/*.png
fi

if [ x"$PrjIcon" = "x" ]; then
  cp %{_datadir}/gambas2/gambas-16x16.png $PrjDir/.icon/16.png
  cp %{_datadir}/gambas2/gambas-32x32.png $PrjDir/.icon/32.png
  cp %{_datadir}/gambas2/gambas-48x48.png $PrjDir/.icon/48.png
elif [ ! -f $PrjDir/$PrjIcon ]; then
  restore_macro
  echo "Project icon not found."
  exit 1
else
  cp $PrjDir/$PrjIcon $PrjDir/.icon/16.png
  cp $PrjDir/$PrjIcon $PrjDir/.icon/32.png
  cp $PrjDir/$PrjIcon $PrjDir/.icon/48.png
fi

cd $PrjDir/../; \
  tar cfvj $HOME/RPM/SOURCES/$PrjStartup-$PrjMajorVers.$PrjMinorVers.$PrjRelVers.tar.bz2 \
    --exclude=*~ \
    --exclude=.lock \
    --exclude=.svn \
    --exclude=*/.xvpics/* \
    --exclude=.xvpics/* \
    --exclude=*APPUNTI* \
    --exclude=*.conf* \
    --exclude=*.ini \
    --exclude=*.xml \
    --exclude=*.sql \
    --exclude=*.ps \
    --exclude=*.txt \
    --exclude=*.sh \
    --exclude=./*.htm \
    --exclude=./*.html \
    --exclude=rpm-make* \
    --exclude=document \
    --exclude=translate \
    --exclude=test \
    --exclude=help \
    $PrjStartup > /dev/null
if [ $? -ne 0 ]; then
  restore_macro
  exit 1
fi

cd $PrjDir/help/../; echo $(pwd); \
  tar cfvj $HOME/RPM/SOURCES/$PrjStartup-help-$PrjMajorVers.$PrjMinorVers.$PrjRelVers.tar.bz2 \
    --exclude=*~ \
    --exclude=.lock \
    --exclude=.svn \
    --exclude=*.conf* \
    --exclude=*.ini \
    --exclude=*.xml \
    --exclude=*.sql \
    --exclude=*.ps \
    --exclude=*.txt \
    --exclude=*.sh \
    --exclude=*.session \
    --exclude=*.webprj \
    --exclude=templates \
    --exclude=toolbars \
    help > /dev/null
if [ $? -ne 0 ]; then
  restore_macro
  exit 1
fi

echo "Creating .spec file."

(
echo "%define name $PrjStartup"
echo "%define version $PrjMajorVers.$PrjMinorVers"
echo "%define release $PrjRelVers$VersionSuffix"
echo
echo "Summary: $PrjTitle"
echo "Name: $PrjStartup"
echo "Version: $PrjMajorVers.$PrjMinorVers"
echo "Release: $PrjRelVers$VersionSuffix"
echo "Source0: $PrjStartup-$PrjMajorVers.$PrjMinorVers.$PrjRelVers$VersionSuffix.tar.bz2"
echo "License: $PrjLicense"
echo "Group: $PrjGroups"
echo "BuildRoot: %{_tmppath}/$PrjStartup-$PrjMajorVers.$PrjMinorVers-$PrjRelVers$VersionSuffix-buildroot"
echo "Prefix: %{_prefix}"
echo "BuildArch: noarch"
Requires="Requires: gambas2-runtime >= $GambasVersion,gambas2-runtime < 2.1"
for lib in `cat $PrjDir/.project | grep "Library=" | awk -F"=" '{print $2}'`; do
  if [ "$lib" = "gb.form" ]; then
    lib="gb.db.form"
  fi
  fil=`echo "gambas2-$lib" | awk -F"." '{for (n=1;n<=NF;n++) if (n==NF) print $(n); else printf $(n)"-";}'`
  Requires="$Requires,$fil >= $GambasVersion,$fil < 2.1"
done
echo $Requires
echo
echo "AutoReqProv: no"
echo
echo "%description"
echo "$PrjDescription"
echo "This program is written in Gambas2, so you will need Gambas2 to be installed."
echo
echo "%prep"
echo "rm -rf %{buildroot}"
echo
echo "%setup -q -n $PrjStartup"
echo
echo "%build"
echo "/usr/bin/gbc2 -a -g -t -p"
echo "gba2"
echo
echo "%install"
echo "install -d %{buildroot}/%{_bindir}"
echo "install -d %{buildroot}/%{_datadir}/$PrjStartup"
echo "install -d %{buildroot}/%{_datadir}/$PrjStartup/images"
#echo "install -d %{buildroot}/%{_datadir}/$PrjStartup/help"
#echo "install -d %{buildroot}/%{_datadir}/$PrjStartup/help/images"
#for lang in en it fr; do
#  echo "install -d %{buildroot}/%{_datadir}/$PrjStartup/help/$lang"
#  echo "install -d %{buildroot}/%{_datadir}/$PrjStartup/help/$lang/images"
#done
echo "install -p $PrjFilename %{buildroot}/%{_datadir}/$PrjStartup/."
echo "install -p images/* %{buildroot}/%{_datadir}/$PrjStartup/images/."
#echo "install -p help/*.* %{buildroot}/%{_datadir}/$PrjStartup/help/."
#echo "install -p help/images/*.* %{buildroot}/%{_datadir}/$PrjStartup/help/images/."
#for lang in en it fr; do
#  echo "install -p help/$lang/*.* %{buildroot}/%{_datadir}/$PrjStartup/help/$lang/."
#  echo "install -p help/$lang/images/*.* %{buildroot}/%{_datadir}/$PrjStartup/help/$lang/images/."
#done
echo "ln -s ../share/$PrjStartup/$PrjFilename %{buildroot}/%{_bindir}/$PrjFilename"
case "$PrjSystems" in
  mandrake)
    echo "install -d %{buildroot}/%{_miconsdir}"
    echo "install -d %{buildroot}/%{_iconsdir}"
    echo "install -d %{buildroot}/%{_liconsdir}"
    echo "install -p .icon/16.png %{buildroot}/%{_miconsdir}/$PrjIcon"
    echo "install -p .icon/32.png %{buildroot}/%{_iconsdir}/$PrjIcon"
    echo "install -p .icon/48.png %{buildroot}/%{_liconsdir}/$PrjIcon"
    echo "install -d %{buildroot}/%{_menudir}"
    echo "cat << EOF > %{buildroot}/%{_menudir}/$PrjStartup"
    cmd="?package($PrjStartup):command=\"/%{_bindir}/$PrjFilename\""
    cmd="$cmd icon=\"$PrjStartup.png\""
    cmd="$cmd needs=\"X11\""
    cmd="$cmd section=\"$PrjMenus\""
    if [ x"$PrjTitle" = "x" ]; then
      cmd="$cmd title=\"$PrjStartup\""
    else
      cmd="$cmd title=\"$PrjTitle\""
    fi
    if [ x"$PrjDescription" = "x" ]; then
      cmd="$cmd longtitle=\"$PrjStartup\""
    else
      cmd="$cmd longtitle=\"$PrjDescription\""
    fi
    cmd="$cmd startup_notify=\"true\""
    echo $cmd
    echo "EOF"
    echo
    echo "%post"
    echo "%{update_menus}"
    echo
    echo "%postun"
    echo "%{clean_menus}"
    ;;
  debian)
    echo "install -d %{buildroot}/%{_datadir}/pixmaps"
    echo "install -p .icon/32.png %{buildroot}/%{_datadir}/pixmaps/$PrjIcon"
    echo "install -d %{buildroot}/%{_datadir}/menu"
    echo "cat << EOF > %{buildroot}/%{_datadir}/menu/$PrjStartup"
    cmd="?package($PrjStartup):command=\"/%{_bindir}/$PrjFilename\""
    cmd="$cmd icon=\"%{_datadir}/pixmaps/$PrjStartup.png\""
    cmd="$cmd needs=\"X11\""
    cmd="$cmd section=\"$PrjMenus\""
    if [ x"$PrjTitle" = "x" ]; then
      cmd="$cmd title=\"$PrjStartup\""
    else
      cmd="$cmd title=\"$PrjTitle\""
    fi
    if [ x"$PrjDescription" = "x" ]; then
      cmd="$cmd longtitle=\"$PrjStartup\""
    else
      cmd="$cmd longtitle=\"$PrjDescription\""
    fi
    cmd="$cmd startup_notify=\"true\""
    echo $cmd
    echo "EOF"
    ;;
  redhat|suse|fedora)
    echo "install -d %{buildroot}/%{_datadir}/pixmaps"
    echo "install -d %{buildroot}/%{_datadir}/applications"
    echo "install -p .icon/48.png %{buildroot}/%{_datadir}/pixmaps/$PrjStartup.png"
    echo "cat << EOF > %{buildroot}/%{_datadir}/applications/$PrjStartup.desktop"
    echo "[Desktop Entry]"
    echo "Version=1.0"
    echo "Encoding=UTF-8"
    echo "Name=$PrjTitle"
    echo "Comment=$PrjDescription"
    echo "Exec=%{_bindir}/$PrjFilename"
    echo "Icon=%{_datadir}/pixmaps/$PrjStartup.png"
    echo "Terminal=False"
    echo "Type=Application"
    echo "StartupNotify=true"
    echo "Categories=X-Red-Hat-Extra;$PrjCategories;"
    echo "EOF"
    ;;
esac
echo
echo "%clean"
echo "rm -rf %{buildroot}"
echo
echo "%files"
echo "%defattr(-,root,root)"
echo "%{_bindir}/$PrjFilename"
echo "%{_datadir}/$PrjStartup/*"
case "$PrjSystems" in
  mandrake)
    echo "/%{_miconsdir}/$PrjIcon"
    echo "/%{_iconsdir}/$PrjIcon"
    echo "/%{_liconsdir}/$PrjIcon"
    echo "/%{_menudir}/$PrjStartup"
    ;;
  debian)
    echo "%{_datadir}/pixmaps/$PrjIcon"
    echo "%{_datadir}/menu/$PrjStartup"
    ;;
  redhat|suse|fedora)
    echo "%{_datadir}/pixmaps/$PrjStartup.png"
    echo "%{_datadir}/applications/$PrjStartup.desktop"
    ;;
esac
echo
echo "%changelog"
if [ -s $PrjDir/ChangeLog ]; then
  cat $PrjDir/ChangeLog
fi
) > $HOME/RPM/SPECS/$PrjStartup.spec

echo "Creating RPM packages."

Temp=`mktemp /tmp/$PrjStartup.build.XXXXXXXXXX`
echo "Make temporary file $Temp."

$RPMBUILD_PROG -ba $HOME/RPM/SPECS/$PrjStartup.spec > $Temp 2>&1

kill_macro

if [ -f $HOME/RPM/RPMS/noarch/$PrjStartup-$PrjMajorVers.$PrjMinorVers-$PrjRelVers.noarch.rpm ]; then
  mv $HOME/RPM/RPMS/noarch/$PrjStartup-$PrjMajorVers.$PrjMinorVers-$PrjRelVers.noarch.rpm \
     $HOME/RPM/RPMS/noarch/$PrjStartup-$PrjMajorVers.$PrjMinorVers.$PrjRelVers.noarch.rpm
fi
if [ -f $HOME/RPM/SRPMS/$PrjStartup-$PrjMajorVers.$PrjMinorVers-$PrjRelVers.src.rpm ]; then
  mv $HOME/RPM/SRPMS/$PrjStartup-$PrjMajorVers.$PrjMinorVers-$PrjRelVers.src.rpm \
     $HOME/RPM/SRPMS/$PrjStartup-$PrjMajorVers.$PrjMinorVers.$PrjRelVers.src.rpm
fi

if [ "$LANG" == "it_IT.UTF-8" ]; then
  msg=`cat $Temp | grep "Scritto:" | grep "noarch.rpm"`
else
  msg=`cat $Temp | grep "Wrote:" | grep "noarch.rpm"`
fi
#msg=`cat $Temp | grep "Wrote:" | grep "noarch.rpm"`
if [ x"$msg" != "x" ]; then
  Package="$PrjStartup-$PrjMajorVers.$PrjMinorVers.$PrjRelVers.noarch.rpm"
  if [ "$PrjSystems" = "debian" ]; then
    cd $HOME/RPM/RPMS/noarch; fakeroot alien --to-deb -k $Package
    Package=$PrjStartup"_"$PrjMajorVers.$PrjMinorVers.$PrjRelVers"_all.deb"
    if [ ! -f $HOME/RPM/RPMS/noarch/$Package ]; then
      restore_macro
      echo "alien command has failed"
      exit 1
    fi
  fi
  Package="$PrjStartup-$PrjMajorVers.$PrjMinorVers.$PrjRelVers.src.rpm"
  if [ "$PrjSystems" = "debian" ]; then
    cd $HOME/RPM/SRPMS; fakeroot alien --to-deb -k $Package
    Package=$PrjStartup"_"$PrjMajorVers.$PrjMinorVers.$PrjRelVers"_all.deb"
    if [ ! -f $HOME/RPM/SRPMS/$Package ]; then
      restore_macro
      echo "alien command has failed"
      exit 1
    fi
    mv $HOME/RPM/SRPMS/$Package $HOME/RPM/SRPMS/$PrjStartup"_"$PrjMajorVers.$PrjMinorVers.$PrjRelVers"_all.src.deb"
  fi
  rm $Temp
  restore_macro
  echo "The packages have been successfully created."
  exit 0
else
  restore_macro
  echo "The package build has failed."
  exit 1
fi

exit 0

